import React, { useState, useEffect, useCallback, useRef } from 'react';
import ModuleTitleWithTabs from '@/pages/Home_v_2504/components/common/ModuleTitleWithTabs';
import SmoothLineChart from '@/components/V2504/SmoothLineChart';
import styles from './index.less';
import { useDataType, useMapState } from '@/pages/Home_v_2504/components/MapStateContext';
import services from '@/services';
import { getHighIncidenceTime } from '@/services/CityProblem';
import { formatDateTime } from '@/pages/Home_v_2504/hooks/useApiControl';
import useLatestValue from '@/pages/Home_v_2504/hooks/useLatestValue';
import usePageAwareApi from '@/pages/Home_v_2504/hooks/usePageAwareApi';

interface HighFrequencyProps {
  dateType?: string;
  dateRange?: { startTime: string; endTime: string };
  dataTypeKey?: string;
  pageId?: string;
}

/**
 * 餐饮油烟高发时段组件
 * 展示餐饮油烟事件的高发时段统计数据
 */
const HighFrequency: React.FC<HighFrequencyProps> = ({ 
  dateType,
  dateRange,
  dataTypeKey = '餐饮油烟事件',
  pageId = 'restaurantOil'
}) => {
  // 地图状态
  const { mapState } = useMapState();
  // 使用ref存储最新的mapState，避免闭包问题
  const mapStateRef = useLatestValue(mapState);
  
  // 数据状态
  const [data, setData] = useState<{ time: string; value: number }[]>([]);
  
  // 转换日期类型为API参数
  const translateDateType = useCallback((type: string) => {
    switch (type) {
      case 'year': return '1';
      case 'month': return '2';
      case 'day': return '3';
      case 'range': return '4';
      default: return '2'; // 默认月视图
    }
  }, []);

  // 参数格式化函数 - 将组件格式的参数转换为API需要的格式
  const formatApiParams = useCallback((params: any) => {
    const formattedParams = { ...params };
    
    // 确保时间字段包含完整的时间部分
    if (formattedParams.time) {
      formattedParams.time = formatDateTime(formattedParams.time);
    }
    
    if (formattedParams.customizeStart) {
      formattedParams.customizeStart = formatDateTime(formattedParams.customizeStart);
    }
    
    if (formattedParams.customizeEnd) {
      formattedParams.customizeEnd = formatDateTime(formattedParams.customizeEnd, true);
    }
    
    return formattedParams;
  }, []);

  // 使用usePageAwareApi管理高发时段API调用
  const highIncidenceTimeApi = usePageAwareApi({
    apiFunction: async (params: any) => {
      const response = await getHighIncidenceTime(params);
      
      if (response.code === 200 && response.data) {
        return response.data;
      }
      
      throw new Error('获取高发时段数据失败');
    },
    onSuccess: (data: any) => {
      // 转换接口返回的数据格式为图表需要的格式
      const formattedData = data.map((item: any) => ({
        time: `${item.name.padStart(2, '0')}:${item.name === '23' ? '59:59' : '00:00'}`,
        value: parseInt(item.value) || 0
      }));
      
      console.log('高发时段数据成功:');
      setData(formattedData);
    },
    onError: (error: any) => {
      console.error('获取高发时段数据出错:', error);
      // 出错时使用空数据
      setData([]);
    },
    formatParams: formatApiParams,
    apiName: '高发时段',
    debounceTime: 300,
    cacheTime: 5000, // 缓存5秒，避免频繁刷新时重复请求
    enableDebug: false,
    pageId: pageId, // 添加页面ID以启用页面感知
  });

  // 构建API参数函数
  const buildApiParams = useCallback(() => {
    const currentMapState = mapStateRef.current;
    
    // 如果没有必要的时间参数，返回null表示不应该发起请求
    if (!currentMapState.rightPanelTime?.startTime || !currentMapState.rightPanelTime?.endTime) {
      return null;
    }
    
    // 获取当前页面的时间设置
    const pageTimeSettings = currentMapState.pageTimeSettings?.[pageId];
    if (!pageTimeSettings) {
      console.log(`缺少${pageId}页面的时间设置，使用全局设置`);
    }
    
    // 使用页面级别的时间设置，如果存在的话
    const rightPanelTime = pageTimeSettings?.rightPanelTime || currentMapState.rightPanelTime;
    
    // 使用当前时间类型，如果不存在则使用props或默认值
    const effectiveType = rightPanelTime.type || dateType || 'month';
    
    // 基础参数对象
    const params: any = {
      type: translateDateType(effectiveType),
      subTypeCode: useDataType(dataTypeKey)
    };
    
    // 处理日期参数
    if (effectiveType === 'range') {
      params.customizeStart = rightPanelTime.startTime;
      params.customizeEnd = rightPanelTime.endTime;
    } else {
      // 年、月、日视图使用time参数
      params.time = rightPanelTime.startTime;
    }
    
    // 处理区域参数
    if (currentMapState.regionCode === '5101') {
      // 成都，这两个参数不传
      params.districtCode = undefined;
      params.streetCode = undefined;
    } else if (currentMapState.regionCode.length === 6) {
      // 区县级别
      params.districtCode = currentMapState.regionCode;
    } else if (currentMapState.regionCode.length === 9) {
      // 街道级别
      params.districtCode = currentMapState.regionCode.substring(0, 6);
      params.streetCode = currentMapState.regionCode;
    }
    
    return params;
  }, [mapStateRef, dateType, translateDateType, dataTypeKey, pageId]);
  
  // 在地图状态变化时加载数据
  useEffect(() => {
    const params = buildApiParams();
    
    // 如果没有必要的参数，不发送请求
    if (!params) {
      console.log('缺少必要的日期参数，暂不加载高发时段数据');
      return;
    }
    
    console.log(`加载${pageId}页面高发时段数据，参数:`, params);
    highIncidenceTimeApi.callApi(params);
    
  }, [mapState.regionCode, mapState.rightPanelTime, mapState.pageTimeSettings, buildApiParams, highIncidenceTimeApi, pageId]);

  return (
    <div className={styles.container}>
      <ModuleTitleWithTabs title="高发时段" />
      <div className={styles.chartContainer}>
        <SmoothLineChart 
          data={data} 
          unitText="单位：件"
          lineColorStop1="rgba(255, 188, 39, 0)"
          lineColorStop2="rgba(255, 188, 39, 0.36)"
          style={{ height: '150px', width: '100%' }}
        />
      </div>
    </div>
  );
};

export default HighFrequency; 