import React, { useEffect, useState, useCallback, useRef } from 'react';
import ModuleTitleWithTabs from '@/pages/Home_v_2504/components/common/ModuleTitleWithTabs';
import ChartWithLegend, { ChartDataItem } from '@/components/V2504/ChartWithLegend';
import styles from './index.less';
import { useDataType, useMapState } from '@/pages/Home_v_2504/components/MapStateContext';
import { getSourceSystemRank } from '@/services/CityProblem';
import { formatDateTime } from '@/pages/Home_v_2504/hooks/useApiControl';
import useLatestValue from '@/pages/Home_v_2504/hooks/useLatestValue';
import classNames from 'classnames';
import usePageAwareApi from '@/pages/Home_v_2504/hooks/usePageAwareApi';

interface SourceSectionProps {
  dateType?: string;
  dateRange?: { startTime: string; endTime: string };
  dataTypeKey?: string;
  isThinLayout?: boolean;
  pageId?: string;
}

const SourceSection: React.FC<SourceSectionProps> = ({ 
  dateType,
  dateRange,
  dataTypeKey = '餐饮油烟事件',
  isThinLayout = false,
  pageId = 'restaurantOil',
}) => {
  // 地图状态
  const { mapState } = useMapState();
  const mapStateRef = useLatestValue(mapState);
  
  // 统计数据
  const [data, setData] = useState<ChartDataItem[]>([]);
  
  // 图表颜色
  const colors = ['rgb(49, 198, 250)', 'rgb(255, 227, 53)', 'rgb(255, 155, 93)', 'rgb(71, 223, 155)', 'rgb(170, 145, 255)', 'rgb(97, 141, 255)'];
  const colorsThin = ['rgb(209, 133, 255)', 'rgb(255, 155, 93)', 'rgb(71, 223, 155)', 'rgb(170, 145, 255)', 'rgb(97, 141, 255)', 'rgb(49, 198, 250)', 'rgb(49, 198, 250)'];
  
  // 转换日期类型为API参数
  const translateDateType = useCallback((type: string) => {
    switch (type) {
      case 'year': return '1';
      case 'month': return '2';
      case 'day': return '3';
      case 'range': return '4';
      default: return '2'; // 默认月视图
    }
  }, []);
  
  // 构建API请求参数
  const buildApiParams = useCallback(() => {
    const currentMapState = mapStateRef.current;
    
    // 如果没有必要的时间参数，返回null表示不应该发起请求
    if (!currentMapState.rightPanelTime?.startTime || !currentMapState.rightPanelTime?.endTime) {
      return null;
    }
    
    // 获取当前页面的时间设置
    const pageTimeSettings = currentMapState.pageTimeSettings?.[pageId];
    if (!pageTimeSettings) {
      console.log(`缺少${pageId}页面的时间设置，使用全局设置`);
    }
    
    // 使用页面级别的时间设置，如果存在的话
    const rightPanelTime = pageTimeSettings?.rightPanelTime || currentMapState.rightPanelTime;
    
    // 使用当前时间类型，如果不存在则使用props或默认值
    const effectiveType = rightPanelTime.type || dateType || 'month';
    
    // 基础参数对象
    const params: any = {
      type: translateDateType(effectiveType)
    };
    
    // 处理日期参数
    if (effectiveType === 'range') {
      params.customizeStart = formatDateTime(rightPanelTime.startTime);
      params.customizeEnd = formatDateTime(rightPanelTime.endTime, true);
    } else {
      // 年、月、日视图使用time参数
      params.time = formatDateTime(rightPanelTime.startTime);
    }
    
    // 处理区域参数
    if (currentMapState.regionCode === '5101') {
      // 成都，这两个参数就不传
      params.districtCode = undefined;
      params.streetCode = undefined;
    } else if (currentMapState.regionCode.length === 6) {
      // 区县级别
      params.districtCode = currentMapState.regionCode;
    } else if (currentMapState.regionCode.length === 9) {
      // 街道级别
      params.districtCode = currentMapState.regionCode.substring(0, 6);
      params.streetCode = currentMapState.regionCode;
    }
    
    // 使用传入的dataTypeKey获取对应的subTypeCode
    params.subTypeCode = useDataType(dataTypeKey);
    
    return params;
  }, [mapStateRef, dateType, translateDateType, dataTypeKey, pageId]);
  
  // 使用usePageAwareApi管理问题来源API调用
  const sourceSystemRankApi = usePageAwareApi({
    apiFunction: async (params: any) => {
      console.log('发起问题来源请求，参数:', params);
      const response = await getSourceSystemRank(params);
      
      if (response.code === 200 && response.data) {
        return response.data;
      }
      
      throw new Error('获取问题来源数据失败');
    },
    onSuccess: (data: any) => {
      if (!data || !Array.isArray(data) || data.length === 0) {
        let useMockData = false;
        if (useMockData) {
          data = [
            { name: '锦江区', value: 456, percent: '97.2' },
            { name: '金牛区金牛区金牛区金牛区金牛区金牛区', value: 400, percent: '83.5' },
            { name: '成华区', value: 322, percent: '70.8' },
            { name: '青羊区', value: 186, percent: '65.6' },
            { name: '双流区', value: 152, percent: '12.9' },
            { name: '武侯区', value: 122, percent: '10.6' },
            { name: '龙泉驿区', value: 121, percent: '10.5' },
            { name: '温江区', value: 120, percent: '10.4' },
            { name: '新都区', value: 119, percent: '10.3' },
            { name: '郫都区', value: 118, percent: '10.2' },
            { name: '都江堰市', value: 117, percent: '10.1' }
          ];
          console.log('使用模拟问题来源数据');
        }else{
          setData([]);
          return;
        }
      }
      
      // 转换接口返回的数据格式为图表需要的格式
      const formattedData = data.map((item: any) => ({
        name: item.name || '未知来源',
        value: parseInt(item.value) || 0,
        percentage: parseFloat(item.percent) || 0
      }));
      
      console.log('问题来源数据成功:', formattedData);
      setData(formattedData);
    },
    onError: (error: any) => {
      console.error('获取问题来源数据出错:', error);
      setData([]);
    },
    apiName: '问题来源',
    debounceTime: 300,
    cacheTime: 5000, // 缓存5秒，避免频繁刷新时重复请求
    enableDebug: false,
    pageId: pageId, // 添加页面ID以启用页面感知
  });

  // 在地图状态变化时加载数据
  useEffect(() => {
    const params = buildApiParams();
    
    // 如果没有必要的参数，不发送请求
    if (!params) {
      console.log('缺少必要的日期参数，暂不加载问题来源数据');
      return;
    }
    
    console.log(`加载${pageId}页面问题来源数据，参数:`, params);
    sourceSystemRankApi.callApi(params);
    
  }, [mapState.regionCode, mapState.rightPanelTime, mapState.pageTimeSettings, buildApiParams, sourceSystemRankApi, pageId]);

  return (
    <div className={styles.container}>
      <ModuleTitleWithTabs title="问题来源" />
      <div className={classNames(styles.content, { [styles.contentThin]: isThinLayout })}>
        <ChartWithLegend 
          data={data}
          colors={isThinLayout ? colorsThin : colors}
          chartWidth={isThinLayout ? 200 : 170}
          chartHeight={isThinLayout ? 200 : 170}
          unit="件"
          showPercentage={true}
          showCount={true}
          isThinLayout={isThinLayout}
          legendHeight={isThinLayout ? 260 : 170}
          fullCountOfLegend={isThinLayout ? 5 : 3}
        />
      </div>
    </div>
  );
};

export default SourceSection; 